
import { Node } from "../node-types/abstract-node";

export abstract class Edge {

    public static readonly EDGE_TYPE: string;
    public static readonly SOURCE_TYPES: string[];
    public static readonly REFERENCE_TYPES: string[];
    public static readonly RELATIONSHIP: string;

    public readonly source: Node;
    public readonly reference: Node;
  
    public constructor(source: Node, reference: Node) {
        this.source = source;
        this.reference = reference;
    }

    public static nodesCompatible(source: Node, reference: Node): boolean {
        const sourceClass = source.constructor as typeof Node;
        const referenceClass = reference.constructor as typeof Node;
        if (!this.SOURCE_TYPES.includes(sourceClass.TYPE)) return false;
        if (!this.REFERENCE_TYPES.includes(referenceClass.TYPE)) return false;
        return true;
    }
  
    public static createEdge(source: Node, reference: Node): Edge {
        throw new Error('You must override the static createEdge method in specific edge implementations');
    }

}