
import { Edge } from "../edge-types/abstract-edge";

export abstract class Node {

	public static readonly EDGE_PATTERN: RegExp = /^- (\S+) \[\[({.+}) (.+)\]\]$/;
	public static readonly TYPE: string;
	public static readonly TYPE_PATTERN: RegExp;
	public static readonly SYMBOL: string;

	public readonly name: string;
	public readonly data: string;
	public readonly outgoingReferences: Edge[] = [];
	public readonly incomingReferences: Edge[] = [];

	public constructor(name: string, data: string) {
		this.name = name;
		this.data = data;
	}

	public static load(fileName: string, fileData: string): Node {
		throw new Error('You must override the static parse method in specific node implementations');
	}

	public parseDataEdges() {
		type EdgeData = {edgeType: string, referenceType: string, referenceName: string};
		const edges: EdgeData[] = [];
		const lines = this.data.split('\n');
		for (const line of lines) { 
			const match = line.match(Node.EDGE_PATTERN);
			if (match) edges.push({
				edgeType: match[1],
				referenceType: match[2],
				referenceName: match[3]
			});
		}
		return edges;
	}

	public registerEdge(edge: Edge) {
		if (edge.source === this) this.outgoingReferences.push(edge);
		else if (edge.reference === this) this.incomingReferences.push(edge);
		else throw new Error('Edge does not reference this node');
	}

}